/*
 * pdftxt - a mupdf-based pdf text extractor
 *
 * Copyright (C) 2012-2015 Ali Gholami Rudi
 *
 * This program is released under the Modified BSD license.
 */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "mupdf/fitz.h"

static void extract_page(fz_text_page *pg, int pnum)
{
	fz_text_block *blk;
	fz_text_line *ln;
	fz_text_span *sp;
	fz_text_char *ch;
	char utf[16];
	int i, j;
	for (i = 0; i < pg->len; i++) {
		if (pg->blocks[i].type != FZ_PAGE_BLOCK_TEXT)
			continue;
		blk = pg->blocks[i].u.text;
		for (ln = blk->lines; ln < blk->lines + blk->len; ln++) {
			printf("%d: ", pnum + 1);
			for (sp = ln->first_span; sp; sp = sp->next) {
				for (ch = sp->text; ch < sp->text + sp->len; ch++) {
					int n = fz_runetochar(utf, ch->c);
					for (j = 0; j < n; j++)
						printf("%c", utf[j]);
				}
			}
			printf("\n");
		}
	}
}

static void extract(char *path)
{
	fz_device *tdev;
	fz_page *page;
	fz_context *ctx;
	fz_document *pdf;
	fz_text_sheet *sheet;
	fz_text_page *text;
	int i;
	ctx = fz_new_context(NULL, NULL, FZ_STORE_DEFAULT);
	fz_register_document_handlers(ctx);
	pdf = fz_open_document(ctx, path);
	if (!pdf)
		return;
	for (i = 0; i < fz_count_pages(ctx, pdf); i++) {
		if (!(page = fz_load_page(ctx, pdf, i)))
			continue;
		text = fz_new_text_page(ctx);
		sheet = fz_new_text_sheet(ctx);
		tdev = fz_new_text_device(ctx, sheet, text);
		fz_run_page(ctx, page, tdev, &fz_identity, NULL);
		extract_page(text, i);
		fz_drop_device(ctx, tdev);
		fz_drop_text_page(ctx, text);
		fz_drop_text_sheet(ctx, sheet);
		fz_drop_page(ctx, page);
	}
	fz_drop_document(ctx, pdf);
	fz_drop_context(ctx);
}

int main(int argc, char *argv[])
{
	int i;
	if (argc < 2) {
		printf("usage: %s files\n", argv[0]);
		return 1;
	}
	for (i = 1; i < argc; i++)
		extract(argv[i]);
	return 0;
}
